@@ -10,11 +10,10 @@ import os, re
 
 from mercurial import util, patch, commands
 from mercurial import match as matchmod
-from hgext import record
 
 from tortoisehg.util import hglib
 from tortoisehg.util.patchctx import patchctx
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import qtlib, qscilib, lexers, visdiff, revert, rejects
 from tortoisehg.hgqt import filelistview, filedata, blockmatcher, manifestmodel
 
@@ -420,7 +419,7 @@ class ChunksWidget(QWidget):
                                 opts=diffopts):
                 buf.write(p)
             buf.seek(0)
-            chunks = record.parsepatch(buf)
+            chunks = patch.parsepatch(buf)
             if chunks:
                 header = chunks[0]
                 return [header] + header.hunks
@@ -746,7 +745,7 @@ class DiffBrowser(QFrame):
         elif type(self._ctx.rev()) is str:
             chunks = self._ctx._files[filename]
         else:
-            header = record.parsepatch(cStringIO.StringIO(fd.diff))[0]
+            header = patch.parsepatch(cStringIO.StringIO(fd.diff))[0]
             chunks = [header] + header.hunks
 
         utext = []
@@ -755,9 +754,7 @@ class DiffBrowser(QFrame):
             chunk.selected = False
             chunk.write(buf)
             chunk.lines = buf.getvalue().splitlines()
-            utext += [l.decode(fd.textEncoding(), 'replace')
-                      for l in chunk.lines]
-            utext.append('')
+            utext.append(buf.getvalue().decode(fd.textEncoding(), 'replace'))
         self.sci.setText(u'\n'.join(utext))
 
         start = 0
