@@ -15,9 +15,9 @@ from PyQt4.QtGui import *
 from mercurial import hg, util, scmutil, httpconnection
 
 from tortoisehg.util import hglib, paths, wconfig
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import cmdcore, cmdui, qtlib, thgrepo
-from tortoisehg.hgqt import hgrcutil, hgemail, rebase, resolve
+from tortoisehg.hgqt import bookmark, hgrcutil, hgemail, rebase, resolve
 
 def parseurl(url):
     assert type(url) == unicode
@@ -46,7 +46,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
 
     switchToRequest = pyqtSignal(QString)
 
-    def __init__(self, repoagent, parent=None, embedded=False):
+    def __init__(self, repoagent, parent=None):
         QWidget.__init__(self, parent)
 
         layout = QVBoxLayout()
@@ -61,11 +61,9 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self._lastbfile = None  # output bundle of last incoming command
         self.opts = {}
         self.cmenu = None
-        self.embedded = embedded
 
         s = QSettings()
-        for opt in ('subrepos', 'force', 'new-branch', 'noproxy', 'debug',
-                    'mq'):
+        for opt in ('force', 'new-branch', 'noproxy', 'debug', 'mq'):
             val = s.value('sync/' + opt, None).toBool()
             if val:
                 if opt != 'mq' or 'mq' in self.repo.extensions():
@@ -76,6 +74,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
                 self.opts[opt] = val
 
         self._repoagent.configChanged.connect(self.reload)
+        self._repoagent.repositoryChanged.connect(self._onRepositoryChanged)
 
         tb = QToolBar(self)
         tb.setIconSize(qtlib.toolBarIconSize())
@@ -104,6 +103,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.pushAction = \
         newaction(_('Push outgoing changes to selected URL'),
              'hg-push', lambda: self.pushclicked(None))
+        newaction(_('Sync Bookmarks'), 'thg-sync-bookmarks', self.syncBookmark)
         newaction(_('Email outgoing changesets for remote repository'),
              'mail-forward', self.emailclicked)
 
@@ -137,9 +137,9 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.targetcombo.setEnabled(False)
         self.targetcheckbox = QCheckBox(_('Target:'))
         self.targetcheckbox.toggled.connect(self.targetcombo.setEnabled)
-        tb.addSeparator().setVisible(embedded)
-        tb.addWidget(self.targetcheckbox).setVisible(embedded)
-        tb.addWidget(self.targetcombo).setVisible(embedded)
+        tb.addSeparator()
+        tb.addWidget(self.targetcheckbox)
+        tb.addWidget(self.targetcombo)
 
         hbox = QHBoxLayout()
         hbox.setContentsMargins(0, 0, 0, 0)
@@ -171,6 +171,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
 
         self.urlentry = QLineEdit()
         self.urlentry.textChanged.connect(self.urlChanged)
+        self.urlentry.returnPressed.connect(self.saveclicked)
         tbar.addWidget(self.urlentry)
 
         # even though currentRowChanged fires pathSelected, clicked signal is
@@ -231,32 +232,33 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         return self._repoagent.rawRepo()
 
     def canswitch(self):
-        return not self.targetcheckbox.isChecked()
+        return False
 
-    def loadTargets(self, ctx):
+    def _loadTargets(self):
         self.targetcombo.clear()
         # itemData(role=UserRole) is the argument list to pass to hg
-        selIndex = 0
-        self.targetcombo.addItem(_('rev: %d (%s)') % (ctx.rev(), str(ctx)),
-                                 ('--rev', str(ctx.rev())))
+        self.targetcombo.addItem('', ('--rev', 'null'))  # placeholder
 
         for name in hglib.namedbranches(self.repo):
             uname = hglib.tounicode(name)
             self.targetcombo.addItem(_('branch: ') + uname, ('--branch', name))
             self.targetcombo.setItemData(self.targetcombo.count() - 1, name,
                                          Qt.ToolTipRole)
-            if ctx.thgbranchhead() and name == ctx.branch():
-                selIndex = self.targetcombo.count() - 1
-        for name in self.repo._bookmarks.keys():
+        for name in sorted(self.repo._bookmarks):
             uname = hglib.tounicode(name)
             self.targetcombo.addItem(_('bookmark: ') + uname,
                                      ('--bookmark', name))
             self.targetcombo.setItemData(self.targetcombo.count() - 1, name,
                                          Qt.ToolTipRole)
-            if name in ctx.bookmarks():
-                selIndex = self.targetcombo.count() - 1
 
-        return selIndex
+    def _findTargetIndex(self, ctx):
+        for name in ctx.bookmarks():
+            uname = hglib.tounicode(name)
+            return self.targetcombo.findText(_('bookmark: ') + uname)
+        if ctx.node() in self.repo.branchheads(ctx.branch()):
+            uname = hglib.tounicode(ctx.branch())
+            return self.targetcombo.findText(_('branch: ') + uname)
+        return 0
 
     def refreshTargets(self, rev):
         if type(rev) is not int:
@@ -266,15 +268,20 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             return
 
         ctx = self.repo.changectx(rev)
-        index = self.loadTargets(ctx)
-
-        if index < 0:
-            index = 0
-        self.targetcombo.setCurrentIndex(index)
+        if self.targetcombo.count() <= 0:
+            self._loadTargets()
+        self.targetcombo.setItemText(0, _('rev: %d (%s)') % (ctx.rev(), ctx))
+        self.targetcombo.setItemData(0, ('--rev', str(ctx.rev())))
+        self.targetcombo.setCurrentIndex(self._findTargetIndex(ctx))
 
     def isTargetSelected(self):
         return self.targetcheckbox.isChecked()
 
+    @pyqtSlot(int)
+    def _onRepositoryChanged(self, flags):
+        if flags & thgrepo.LogChanged:
+            self._loadTargets()
+
     def editOptions(self):
         dlg = OptionsDialog(self._repoagent, self.opts, self)
         dlg.setWindowFlags(Qt.Sheet)
@@ -439,7 +446,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
                 (_('Copy &Path'), self.copypath, ''),
                 separator,
                 (_('&Edit...'), self.editurl, 'general'),
-                (_('&Remove...'), self.removeurl, 'menudelete')):
+                (_('&Remove...'), self.removeurl, 'hg-strip')):
                 if text is None:
                     menu.addSeparator()
                     continue
@@ -602,7 +609,8 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         if not self.opts.get('mq'):
             cmdline.append(lurl)
         ucmdline = map(hglib.tounicode, cmdline)
-        overlay = ucmdline[0] not in ('incoming', 'pull')  # no incoming bundle
+        # bypass overlay of incoming bundle to pull changes
+        overlay = ucmdline[0] not in ('fetch', 'incoming', 'pull')
         self._cmdsession = sess = self._repoagent.runCommand(ucmdline, self,
                                                              overlay=overlay)
         sess.commandFinished.connect(self._updateUi)
@@ -646,7 +654,10 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             return
         save = self.currentUrl()
         orev = self.opts.get('rev')
-        self.setEditUrl(bundle)
+        # XXX hack to ignore incoming bundle because it can't apply phase
+        # movement, pull bookmarks and largefiles. further cleanups should
+        # go on default branch.
+        self.setEditUrl(bsource or bundle)
         if rev is not None:
             self.opts['rev'] = str(rev)
         self.pullclicked(bsource)
@@ -666,8 +677,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def inclicked(self):
         url = self.currentUrl()
         link = self.linkifyWithTarget(url)
-        if self.embedded and not url.startswith('p4://') and \
-           not self.opts.get('subrepos'):
+        if not url.startswith('p4://'):
             bfile = hglib.fromunicode(url)
             for badchar in (':', '*', '\\', '?', '#'):
                 bfile = bfile.replace(badchar, '')
@@ -680,7 +690,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         else:
             self._lastbfile = None
             cmdline = ['incoming']
-            sess = self.run(cmdline, ('force', 'branch', 'rev', 'subrepos'))
+            sess = self.run(cmdline, ('force', 'branch', 'rev'))
             sess.commandFinished.connect(self._onIncomingFinished)
 
         self.showMessage.emit(_('Getting incoming changesets from %s...')
@@ -748,16 +758,10 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
 
     def outclicked(self):
         link = self.linkifyWithTarget(self.currentUrl())
-        if self.embedded and not self.opts.get('subrepos'):
-            cmdline = ['outgoing', '--template', '{node}\n']
-            sess = self.run(cmdline, ('force', 'branch', 'rev'))
-            sess.setCaptureOutput(True)
-            sess.commandFinished.connect(self._onOutgoingFinished)
-        else:
-            cmdline = ['outgoing']
-            sess = self.run(cmdline, ('force', 'branch', 'rev', 'subrepos'))
-            sess.commandFinished.connect(self._onOutgoingFinished)
-
+        cmdline = ['outgoing', '--template', '{node}\n']
+        sess = self.run(cmdline, ('force', 'branch', 'rev'))
+        sess.setCaptureOutput(True)
+        sess.commandFinished.connect(self._onOutgoingFinished)
         self.showMessage.emit(_('Finding outgoing changesets to %s...') % link)
 
     @pyqtSlot(int)
@@ -766,13 +770,9 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         if ret == 0:
             data = str(self._cmdsession.readAll())
             nodes = _extractnodeids(data)
-            if nodes:
-                self.showMessage.emit(_('%d outgoing changesets to %s') %
-                                      (len(nodes), link))
-                self.outgoingNodes.emit(nodes)
-            else:
-                self.showMessage.emit(_('outgoing changesets to %s found')
-                                      % link)
+            self.showMessage.emit(_('%d outgoing changesets to %s') %
+                                  (len(nodes), link))
+            self.outgoingNodes.emit(nodes)
         elif ret == 1:
             self.showMessage.emit(_('No outgoing changesets to %s') % link)
         else:
@@ -931,6 +931,11 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         sess.commandFinished.connect(self._onOutgoingEmailFinished)
         self.showMessage.emit(_('Determining outgoing changesets to email...'))
 
+    def syncBookmark(self):
+        dlg = bookmark.SyncBookmarkDialog(self._repoagent, self.currentUrl(),
+                                          self)
+        dlg.exec_()
+
     @pyqtSlot(int)
     def _onOutgoingEmailFinished(self, ret):
         if ret == 0:
@@ -962,7 +967,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         if bundlefile:
             # Set the pull source to the selected bundle file
             self.urlentry.setText(bundlefile)
-            # Execute the incomming command, which will show the revisions in
+            # Execute the incoming command, which will show the revisions in
             # the bundle, and let the user accept or reject them
             self.inclicked()
 
@@ -988,52 +993,6 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.reload()
 
 
-class SyncDialog(QDialog):
-
-    def __init__(self, repoagent, parent=None):
-        super(SyncDialog, self).__init__(parent)
-        self.setWindowFlags(Qt.Window)
-        self.setWindowTitle(_('TortoiseHg Sync'))
-        self.setWindowIcon(qtlib.geticon('thg-sync'))
-        self.resize(850, 550)
-
-        layout = QVBoxLayout()
-        layout.setContentsMargins(5, 5, 5, 0)
-        self.setLayout(layout)
-
-        self._syncwidget = sync = SyncWidget(repoagent, self)
-        layout.addWidget(sync)
-
-        self._cmdlog = cmdui.LogWidget(self)
-        self._stbar = cmdui.ThgStatusBar(self)
-        layout.addWidget(self._cmdlog)
-        layout.addWidget(self._stbar)
-        self._cmdlog.hide()
-        sync.newCommand.connect(self._handleNewCommand)
-        sync.showMessage.connect(self._stbar.showMessage)
-
-    def currentUrl(self):
-        return self._syncwidget.currentUrl()
-
-    def setUrl(self, newurl):
-        self._syncwidget.setUrl(newurl)
-
-    @pyqtSlot(cmdcore.CmdSession)
-    def _handleNewCommand(self, sess):
-        sess.commandFinished.connect(self._stbar.clearProgress)
-        sess.outputReceived.connect(self._cmdlog.appendLog)
-        sess.progressReceived.connect(self._stbar.setProgress)
-        self._cmdlog.show()
-
-    def reject(self):
-        if not self._syncwidget.canExit():
-            if not qtlib.QuestionMsgBox(_('TortoiseHg Sync'),
-                _('Are you sure that you want to cancel synchronization?'),
-                parent=self):
-                return
-        super(SyncDialog, self).reject()
-
-
 class PostPullDialog(QDialog):
     def __init__(self, repoagent, parent):
         super(PostPullDialog, self).__init__(parent)
@@ -1279,34 +1238,23 @@ class SaveDialog(QDialog):
                                 and self.urlentry.text()))
 
 
+def _addBrowseButton(edit, slot):
+    button = QPushButton(_('Browse...'))
+    button.setAutoDefault(False)
+    button.clicked.connect(slot)
+    hbox = QHBoxLayout()
+    hbox.addWidget(edit)
+    hbox.addWidget(button)
+    return hbox
+
+
 class SecureDialog(QDialog):
     def __init__(self, repoagent, urlu, parent):
         super(SecureDialog, self).__init__(parent)
         self._repoagent = repoagent
+        self._querysess = cmdcore.nullCmdSession()
         repo = repoagent.rawRepo()
-
-        def genfingerprint():
-            if u.port is None:
-                portnum = 443
-            else:
-                try:
-                    portnum = int(u.port)
-                except ValueError:
-                    qtlib.WarningMsgBox(_('Certificate Query Error'),
-                                        _('Invalid port number: %s')
-                                        % hglib.tounicode(u.port), parent=self)
-                    return
-            try:
-                pem = ssl.get_server_certificate( (u.host, portnum) )
-                der = ssl.PEM_cert_to_DER_cert(pem)
-            except Exception, e:
-                qtlib.WarningMsgBox(_('Certificate Query Error'),
-                                    hglib.tounicode(str(e)), parent=self)
-                return
-            hash = util.sha1(der).hexdigest()
-            pretty = ":".join([hash[x:x + 2] for x in xrange(0, len(hash), 2)])
-            le.setText(pretty)
-
+        self._url = urlu
         u = parseurl(urlu)
         assert u.host
         uhost = hglib.tounicode(u.host)
@@ -1350,65 +1298,66 @@ class SecureDialog(QDialog):
         if hasattr(le, 'setPlaceholderText'): # Qt >= 4.7
             le.setPlaceholderText(_('### host certificate fingerprint ###'))
         hbox.addWidget(le)
-        try:
-            import ssl # Python 2.6 or backport for 2.5
-            qb = QPushButton(_('Query'))
-            qb.clicked.connect(genfingerprint)
-            qb.setEnabled(False)
-            self.fprintradio.toggled.connect(qb.setEnabled)
-            hbox.addWidget(qb)
-        except ImportError:
-            pass
+        self._querybutton = qb = QPushButton(_('Query'))
+        qb.clicked.connect(self._queryFingerprint)
+        self.fprintradio.toggled.connect(self._updateUi)
+        hbox.addWidget(qb)
         vbox.addWidget(self.cacertradio)
         vbox.addWidget(self.fprintradio)
         vbox.addLayout(hbox)
         vbox.addWidget(self.insecureradio)
 
-        self.cacertradio.setEnabled(bool(repo.ui.config('web', 'cacerts')))
         self.cacertradio.setChecked(True) # default
         if fprint:
             self.fprintradio.setChecked(True)
         elif repo.ui.config('insecurehosts', u.host):
             self.insecureradio.setChecked(True)
 
+        self._authentries = {}  # key: QLineEdit
         authbox = QGroupBox(_('User Authentication'))
         form = QFormLayout()
         authbox.setLayout(form)
         self.layout().addWidget(authbox)
 
-        self.userentry = QLineEdit(u.user or auth.get('username', ''))
-        self.userentry.setToolTip(
+        k = 'username'
+        self._authentries[k] = e = QLineEdit(u.user or auth.get(k, ''))
+        e.setToolTip(
 _('''Optional. Username to authenticate with. If not given, and the remote
 site requires basic or digest authentication, the user will be prompted for
 it. Environment variables are expanded in the username letting you do
 foo.username = $USER.'''))
-        form.addRow(_('Username'), self.userentry)
+        form.addRow(_('Username'), e)
 
-        self.pwentry = QLineEdit(u.passwd or auth.get('password', ''))
-        self.pwentry.setEchoMode(QLineEdit.Password)
-        self.pwentry.setToolTip(
+        k = 'password'
+        self._authentries[k] = e = QLineEdit(u.passwd or auth.get(k, ''))
+        e.setEchoMode(QLineEdit.Password)
+        e.setToolTip(
 _('''Optional. Password to authenticate with. If not given, and the remote
 site requires basic or digest authentication, the user will be prompted for
 it.'''))
-        form.addRow(_('Password'), self.pwentry)
+        form.addRow(_('Password'), e)
         if 'mercurial_keyring' in repo.extensions():
-            self.pwentry.clear()
-            self.pwentry.setEnabled(False)
-            self.pwentry.setToolTip(_('Mercurial keyring extension is enabled. '
-                 'Passwords will be stored in a platform-native '
-                 'secure method.'))
+            e.clear()
+            e.setEnabled(False)
+            e.setToolTip(_('Mercurial keyring extension is enabled. '
+                           'Passwords will be stored in a platform-native '
+                           'secure method.'))
 
-        self.keyentry = QLineEdit(auth.get('key', ''))
-        self.keyentry.setToolTip(
+        k = 'key'
+        self._authentries[k] = e = QLineEdit(auth.get(k, ''))
+        e.setToolTip(
 _('''Optional. PEM encoded client certificate key file. Environment variables
 are expanded in the filename.'''))
-        form.addRow(_('User Certificate Key File'), self.keyentry)
+        form.addRow(_('User Certificate Key'),
+                    _addBrowseButton(e, self._browseClientKey))
 
-        self.chainentry = QLineEdit(auth.get('cert', ''))
-        self.chainentry.setToolTip(
+        k = 'cert'
+        self._authentries[k] = e = QLineEdit(auth.get(k, ''))
+        e.setToolTip(
 _('''Optional. PEM encoded client certificate chain file. Environment variables
 are expanded in the filename.'''))
-        form.addRow(_('User Certificate Chain File'), self.chainentry)
+        form.addRow(_('User Certificate Chain'),
+                    _addBrowseButton(e, self._browseClientCert))
 
         BB = QDialogButtonBox
         bb = QDialogButtonBox(BB.Help|BB.Save|BB.Cancel)
@@ -1418,12 +1367,50 @@ are expanded in the filename.'''))
         self.bb = bb
         self.layout().addWidget(bb)
 
-        self.userentry.selectAll()
-        QTimer.singleShot(0, lambda:self.userentry.setFocus())
+        self._updateUi()
+        e = self._authentries['username']
+        e.selectAll()
+        QTimer.singleShot(0, e.setFocus)
+
+    @pyqtSlot()
+    def _queryFingerprint(self):
+        cmdline = hglib.buildcmdargs('debuggethostfingerprint', self._url)
+        self._querysess = sess = self._repoagent.runCommand(cmdline, self)
+        sess.setCaptureOutput(True)
+        sess.commandFinished.connect(self._onQueryFingerprintFinished)
+        self._updateUi()
+
+    @pyqtSlot(int)
+    def _onQueryFingerprintFinished(self, ret):
+        sess = self._querysess
+        if ret == 0:
+            data = str(sess.readAll())
+            self.fprintentry.setText(hglib.tounicode(data).strip())
+        else:
+            cmdui.errorMessageBox(sess, self, _('Certificate Query Error'))
+        self._updateUi()
 
     def keyringHelp(self):
         qtlib.openhelpcontents('sync.html#security')
 
+    @pyqtSlot()
+    def _browseClientKey(self):
+        e = self._authentries['key']
+        n = QFileDialog.getOpenFileName(
+            self, _('Select User Certificate Key File'), e.text(),
+            ';;'.join([_('PEM files (*.pem *.key)'), _('All files (*)')]))
+        if n:
+            e.setText(n)
+
+    @pyqtSlot()
+    def _browseClientCert(self):
+        e = self._authentries['cert']
+        n = QFileDialog.getOpenFileName(
+            self, _('Select User Certificate Chain File'), e.text(),
+            ';;'.join([_('PEM files (*.pem *.crt *.cer)'), _('All files (*)')]))
+        if n:
+            e.setText(n)
+
     def accept(self):
         path = scmutil.userrcpath()
         fn, cfg = hgrcutil.loadIniFile(path, self)
@@ -1452,16 +1439,10 @@ are expanded in the filename.'''))
         setorclear('hostfingerprints', self.host, fprint)
         setorclear('insecurehosts', self.host, insecure)
 
-        username = hglib.fromunicode(self.userentry.text())
-        password = hglib.fromunicode(self.pwentry.text())
-        key = hglib.fromunicode(self.keyentry.text())
-        chain = hglib.fromunicode(self.chainentry.text())
-
         cfg.set('auth', self.alias+'.prefix', self.host)
-        setorclear('auth', self.alias+'.username', username)
-        setorclear('auth', self.alias+'.password', password)
-        setorclear('auth', self.alias+'.key', key)
-        setorclear('auth', self.alias+'.cert', chain)
+        for k in ['username', 'password', 'key', 'cert']:
+            setorclear('auth', '%s.%s' % (self.alias, k),
+                       hglib.fromunicode(self._authentries[k].text()))
         setorclear('auth', self.alias+'.schemes', self.schemes)
 
         try:
@@ -1472,6 +1453,11 @@ are expanded in the filename.'''))
                                 hglib.tounicode(str(e)), parent=self)
         super(SecureDialog, self).accept()
 
+    @pyqtSlot()
+    def _updateUi(self):
+        self._querybutton.setEnabled(self.fprintradio.isChecked()
+                                     and self._querysess.isFinished())
+
 
 class PathsTree(QTreeView):
     removeAlias = pyqtSignal(QString)
@@ -1584,11 +1570,6 @@ class OptionsDialog(QDialog):
         self.forcecb.setChecked(opts.get('force', False))
         layout.addWidget(self.forcecb)
 
-        self.subrepocb = QCheckBox(
-            _('Recurse into subrepositories') + u' (--subrepos)')
-        self.subrepocb.setChecked(opts.get('subrepos', False))
-        layout.addWidget(self.subrepocb)
-
         repo = repoagent.rawRepo()
         self.noproxycb = QCheckBox(
             _('Temporarily disable configured HTTP proxy'))
@@ -1635,7 +1616,6 @@ class OptionsDialog(QDialog):
                          ('branch', self.branchle)):
             outopts[name] = hglib.fromunicode(le.text()).strip()
 
-        outopts['subrepos'] = self.subrepocb.isChecked()
         outopts['force'] = self.forcecb.isChecked()
         outopts['new-branch'] = self.newbranchcb.isChecked()
         outopts['noproxy'] = self.noproxycb.isChecked()
